body{-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif;margin:0}code{font-family:source-code-pro,Menlo,Monaco,Consolas,Courier New,monospace}.App{text-align:center}.App-logo{height:40vmin;pointer-events:none}@media (prefers-reduced-motion:no-preference){.App-logo{animation:App-logo-spin 20s linear infinite}}.App-header{align-items:center;background-color:#282c34;color:#fff;display:flex;flex-direction:column;font-size:calc(10px + 2vmin);justify-content:center;min-height:100vh}.App-link{color:#61dafb}@keyframes App-logo-spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}.loader{animation:load8 1.1s linear infinite;border:1.1em solid rgba(0,0,0,.2);border-left-color:#000;border-radius:50%;height:6em;left:calc(50% - 4em);position:absolute;top:calc(50% - 4em);transition:opacity .3s;width:6em}.loader--hide{opacity:0}@keyframes load8{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}.code-snippet{background:#eee;border-radius:3px;margin-left:2rem;margin-right:2rem;margin-top:2rem;overflow:auto;padding:1rem}.code-snippet .code{word-wrap:normal;line-height:1.5;text-align:left;white-space:pre;word-break:normal;word-spacing:normal}
/*# sourceMappingURL=main.3d3e8f8a2b0ea7a5bf69.css.map*/